// -*- mode: js2; indent-tabs-mode: nil; js2-basic-offset: 4 -*-
/*

    Copyright (C) 2012-2017  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Lang = imports.lang;

const Main = imports.ui.main;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;

const Gettext = imports.gettext;
const _ = Gettext.domain(Convenience.CURRENT_GETTEXT_DOMAIN).gettext;

function _getBluetooth() {
    if (Convenience.checkGSV(3, 4)) {
        try { return Main.panel._statusArea.bluetooth; } catch(e) { return null; }
    } else if (Convenience.checkGSV(3, 6) || Convenience.checkGSV(3, 8)) {
        try { return Main.panel.statusArea.bluetooth; } catch(e) { return null; }
    } else {
        try { return Main.panel.statusArea.aggregateMenu._bluetooth; } catch(e) { return null; }
    }
}

const GNOME_BLUETOOTH = _getBluetooth();

const KEY_SHOW_BLUETOOTH = 'show-bluetooth';

const PANELMENU_BUTTON_DEFAULT = 0;
const PANELMENU_BUTTON_SHOW = 1;

let _prototypeSync;
let _prototypeUpdateKillswitch;

function _overrideFunction(enabled) {
    if (enabled) {
        if (Convenience.checkGSV(3, 4) || Convenience.checkGSV(3, 6) || Convenience.checkGSV(3, 8)) {
            _prototypeUpdateKillswitch = imports.ui.status.bluetooth.Indicator.prototype._updateKillswitch;
            GNOME_BLUETOOTH._updateKillswitch = function() {
                _prototypeUpdateKillswitch.call(this);
                let pmShowBluetooth = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA).get_enum(KEY_SHOW_BLUETOOTH);
                if (pmShowBluetooth != PANELMENU_BUTTON_DEFAULT) this.actor.visible = pmShowBluetooth == PANELMENU_BUTTON_SHOW;
            };
        } else {
            _prototypeSync = imports.ui.status.bluetooth.Indicator.prototype._sync;
            GNOME_BLUETOOTH._sync = function() {
                _prototypeSync.call(this);
                let pmShowBluetooth = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA).get_enum(KEY_SHOW_BLUETOOTH);
                if (pmShowBluetooth == PANELMENU_BUTTON_DEFAULT) {
                    this._indicator.icon_name = 'bluetooth-active-symbolic';
                    this._item.menu._getMenuItems()[0].actor.show();
                } else {
                    let visible = this._indicator.visible;
                    this._item.menu._getMenuItems()[0].actor.visible = visible;
                    if (Convenience.checkGSV(3, 10) && !visible) this._item.status.text = _("Off");
                    this._indicator.icon_name = visible ? 'bluetooth-active-symbolic' : 'bluetooth-disabled-symbolic';
                    this._indicator.visible = pmShowBluetooth == PANELMENU_BUTTON_SHOW;
                    this._item.actor.visible = this._indicator.visible;
                }
            };
        }
    } else {
        if (Convenience.checkGSV(3, 4) || Convenience.checkGSV(3, 6) || Convenience.checkGSV(3, 8)) {
            GNOME_BLUETOOTH._updateKillswitch = _prototypeUpdateKillswitch;
        } else {
            GNOME_BLUETOOTH._sync = _prototypeSync;
            GNOME_BLUETOOTH._indicator.icon_name = 'bluetooth-active-symbolic';
            GNOME_BLUETOOTH._item.menu._getMenuItems()[0].actor.show();
        }
    }
}

const BTIndicator = new Lang.Class({
    Name: 'BTIndicator',

    _init: function() {
        _overrideFunction(true);
        this._settings =  Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = this._settings.connect('changed::'+KEY_SHOW_BLUETOOTH, Lang.bind(this, this._updateVisibility));
        this._updateVisibility();
    },

    _updateVisibility: function(showBluetooth) {
        if (Convenience.checkGSV(3, 4) || Convenience.checkGSV(3, 6) || Convenience.checkGSV(3, 8)) {
            GNOME_BLUETOOTH._updateKillswitch();
        } else {
            GNOME_BLUETOOTH._sync();
        }
    },

    destroy: function() {
        _overrideFunction(false);
        this._settings.disconnect(this._connectSettings);
        this._updateVisibility();
    }
});

function getExtension() {
    return GNOME_BLUETOOTH ? new BTIndicator() : null;
}
