// -*- mode: js2; indent-tabs-mode: nil; js2-basic-offset: 4 -*-
/*

    Copyright (C) 2012-2017  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Lang = imports.lang;

const Main = imports.ui.main;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;

function _getPanelManagerItems() {
    let items = [
        'showDesktop',
        'favorites',
        'places',
        'applications',
        'activities',
        'hotCorners',
        'dash',
        'dateMenu',
        'recycleBin',
        'notifications',
        'accessibility',
        'keyboard',
        'volume'
    ];
    if (Convenience.checkGSV(3, 4) || Convenience.checkGSV(3, 6) || Convenience.checkGSV(3, 8)) {
        items.push('bluetooth');
        items.push('network');
        items.push('battery');
        items.push('userMenu');
        if (Convenience.checkGSV(3, 4)) items.push('messageTrayAppTitle');
    } else {
        items.push('brightness');
        items.push('network');
        items.push('bluetooth');
        if (!Convenience.checkGSV(3, 10)) items.push('location');
        items.push('airplaneMode');
        items.push('battery');
        items.push('system');
        items.push('aggregateMenu');
        items.push('dropDownArrow');
    }
    return items;
}

const PANELMANAGER_ITEMS = _getPanelManagerItems();

const PanelManager = new Lang.Class({
    Name: 'PanelManager',

    _init: function() {
        for (let i = 0; i < PANELMANAGER_ITEMS.length; i++) {
            try { this[PANELMANAGER_ITEMS[i]] = Me.imports[PANELMANAGER_ITEMS[i]].getExtension(); } catch(e) {}
        }
    },

    destroy: function() {
        for (let i = PANELMANAGER_ITEMS.length - 1; i >= 0; i--) {
            if (this[PANELMANAGER_ITEMS[i]] != null) this[PANELMANAGER_ITEMS[i]].destroy();
            delete this[PANELMANAGER_ITEMS[i]];
        }
    }
});

function init() {
    Convenience.initTranslations();
}

let _extension;

function enable() {
    _extension = new PanelManager();
}

function disable() {
    _extension.destroy();
}
