// -*- mode: js2; indent-tabs-mode: nil; js2-basic-offset: 4 -*-
/*

    Copyright (C) 2012-2017  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Lang = imports.lang;

const Main = imports.ui.main;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;

const KEY_SHOW_MESSAGETRAY_APPTITLE = 'show-messagetray-apptitle';

const MessageTrayAppTitle = new Lang.Class({
    Name: 'MessageTrayAppTitle',

    _init: function() {
        this._settings = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = this._settings.connect('changed::'+KEY_SHOW_MESSAGETRAY_APPTITLE, Lang.bind(this, this._onShowMessageTrayAppTitle));
        this._connectMessageTray = Main.messageTray.actor.connect('notify::hover', Lang.bind(this, this._onShowMessageTrayAppTitle));
        this._onShowMessageTrayAppTitle();
    },

    _onShowMessageTrayAppTitle: function() {
        this._updateVisibility(this._settings.get_boolean(KEY_SHOW_MESSAGETRAY_APPTITLE));
    },

    _updateVisibility: function(visible) {
        Main.messageTray._summaryItems.forEach(function(item) {
            if (item._sourceTitleBin) item._sourceTitleBin.visible = visible;
        });
    },

    destroy: function() {
        Main.messageTray.actor.disconnect(this._connectMessageTray);
        this._settings.disconnect(this._connectSettings);
        this._updateVisibility(true);
    }
});

function getExtension() {
    return Convenience.checkGSV(3, 4) ? new MessageTrayAppTitle() : null;
}
